/* xlftab.c - xlisp function table */
#include "xlset.h"
#include "xlftab.h"
#include "xlbfun.h"
#include "xlcont.h"
#include "xlfio.h"
#include "xlisp.h"
#include "xllist.h"
#include "xlmath.h"
#include "xlstr.h"
#include "xlsys.h"


struct fdef ftab[] = {

	/* evaluator functions */
	{	"eval",		SUBR,	CC65FIX xeval 		},
	{	"apply",	SUBR,	CC65FIX xapply 		},
	{	"funcall",	SUBR,	CC65FIX xfuncall 	},
	{	"quote",	FSUBR,	CC65FIX xquote 		},

	/* symbol functions */
	{	"set",		SUBR,	CC65FIX xset 		},
	{	"setq",		FSUBR,	CC65FIX xsetq 		},
	{	"defun",	FSUBR,	CC65FIX xdefun 		},
	{	"ndefun",	FSUBR,	CC65FIX xndefun 	},
	{	"gensym",	SUBR,	CC65FIX xgensym 	},
	{	"intern",	SUBR,	CC65FIX xintern 	},
	{	"symbol-name",	SUBR,	CC65FIX xsymname 	},
	{	"symbol-plist",	SUBR,	CC65FIX xsymplist 	},
	{	"get",		SUBR,	CC65FIX xget 		},
	{	"putprop",	SUBR,	CC65FIX xputprop 	},
	{	"remprop",	SUBR,	CC65FIX xremprop 	},

	/* list functions */
	{	"car",		SUBR,	CC65FIX xcar 		},
	{	"caar",		SUBR,	CC65FIX xcaar 		},
	{	"cadr",		SUBR,	CC65FIX xcadr 		},
	{	"cdr",		SUBR,	CC65FIX xcdr 		},
	{	"cdar",		SUBR,	CC65FIX xcdar 		},
	{	"cddr",		SUBR,	CC65FIX xcddr 		},
	{	"cons",		SUBR,	CC65FIX xcons 		},
	{	"list",		SUBR,	CC65FIX xlist 		},
	{	"append",	SUBR,	CC65FIX xappend 	},
	{	"reverse",	SUBR,	CC65FIX xreverse 	},
	{	"last",		SUBR,	CC65FIX xlast 		},
	{	"nth",		SUBR,	CC65FIX xnth 		},
	{	"nthcdr",	SUBR,	CC65FIX xnthcdr 	},
	{	"member",	SUBR,	CC65FIX xmember 	},
	{	"memq",		SUBR,	CC65FIX xmemq 		},
	{	"assoc",	SUBR,	CC65FIX xassoc 		},
	{	"assq",		SUBR,	CC65FIX xassq 		},
	{	"subst",	SUBR,	CC65FIX xsubst 		},
	{	"sublis",	SUBR,	CC65FIX xsublis 	},
	{	"length",	SUBR,	CC65FIX xlength 	},
	{	"mapcar",	SUBR,	CC65FIX xmapcar 	},
	{	"maplist",	SUBR,	CC65FIX xmaplist 	},

	/* destructive list functions */
	{	"rplaca",	SUBR,	CC65FIX xrplca 		},
	{	"rplacd",	SUBR,	CC65FIX xrplcd 		},
	{	"nconc",	SUBR,	CC65FIX xnconc 		},
	{	"delete",	SUBR,	CC65FIX xdelete 	},
	{	"delq",		SUBR,	CC65FIX xdelq 		},

	/* predicate functions */
	{	"atom",		SUBR,	CC65FIX xatom 		},
	{	"symbolp",	SUBR,	CC65FIX xsymbolp 	},
	{	"numberp",	SUBR,	CC65FIX xnumberp 	},
	{	"boundp",	SUBR,	CC65FIX xboundp 	},
	{	"null",		SUBR,	CC65FIX xnull 		},
	{	"not",		SUBR,	CC65FIX xnull 		},
	{	"listp",	SUBR,	CC65FIX xlistp 		},
	{	"consp",	SUBR,	CC65FIX xconsp 		},
	{	"eq",		SUBR,	CC65FIX xeq 		},
	{	"equal",	SUBR,	CC65FIX xequal 		},

	/* control functions */
	{	"cond",		FSUBR,	CC65FIX xcond 		},
	{	"and",		FSUBR,	CC65FIX xand 		},
	{	"or",		FSUBR,	CC65FIX xor 		},
	{	"let",		FSUBR,	CC65FIX xlet 		},
	{	"if",		FSUBR,	CC65FIX xif 		},
	{	"progn",	FSUBR,	CC65FIX xprogn 		},
	{	"while",	FSUBR,	CC65FIX xwhile 		},
	{	"repeat",	FSUBR,	CC65FIX xrepeat 	},

	/* arithmetic functions */
	{	"+",		SUBR,	CC65FIX xadd 		},
	{	"-",		SUBR,	CC65FIX xsub 		},
	{	"*",		SUBR,	CC65FIX xmul 		},
	{	"/",		SUBR,	CC65FIX xdiv 		},
	{	"1+",		SUBR,	CC65FIX xadd1 		},
	{	"1-",		SUBR,	CC65FIX xsub1 		},
	{	"rem",		SUBR,	CC65FIX xrem 		},
	{	"minus",	SUBR,	CC65FIX xminus 		},
	{	"min",		SUBR,	CC65FIX xmin 		},
	{	"max",		SUBR,	CC65FIX xmax 		},
	{	"abs",		SUBR,	CC65FIX xabs 		},

	/* bitwise logical functions */
	{	"bit-and",	SUBR,	CC65FIX xbitand 	},
	{	"bit-ior",	SUBR,	CC65FIX xbitior 	},
	{	"bit-xor",	SUBR,	CC65FIX xbitxor 	},
	{	"bit-not",	SUBR,	CC65FIX xbitnot 	},

	/* numeric comparison functions */
	{	"<",		SUBR,	CC65FIX xlss 		},
	{	"<=",		SUBR,	CC65FIX xleq 		},
	{	"=",		SUBR,	CC65FIX xeql 		},
	{	"/=",		SUBR,	CC65FIX xneq 		},
	{	">=",		SUBR,	CC65FIX xgeq 		},
	{	">",		SUBR,	CC65FIX xgtr 		},

	/* string functions */
	{	"strlen",	SUBR,	CC65FIX xstrlen 	},
	{	"strcat",	SUBR,	CC65FIX xstrcat 	},
	{	"substr",	SUBR,	CC65FIX xsubstr 	},
	{	"ascii",	SUBR,	CC65FIX xascii 		},
	{	"chr",		SUBR,	CC65FIX xchr 		},
	{	"atoi",		SUBR,	CC65FIX xatoi 		},
	{	"itoa",		SUBR,	CC65FIX xitoa 		},

	/* I/O functions */
	{	"read",		SUBR,	CC65FIX xread 		},
	{	"print",	SUBR,	CC65FIX xprint 		},
	{	"prin1",	SUBR,	CC65FIX xprin1 		},
	{	"princ",	SUBR,	CC65FIX xprinc 		},
	{	"terpri",	SUBR,	CC65FIX xterpri 	},
	{	"flatsize",	SUBR,	CC65FIX xflatsize 	},
	{	"flatc",	SUBR,	CC65FIX xflatc 		},
	{	"explode",	SUBR,	CC65FIX xexplode 	},
	{	"explodec",	SUBR,	CC65FIX xexplc 		},
	{	"implode",	SUBR,	CC65FIX ximplode 	},
	{	"maknam",	SUBR,	CC65FIX xmaknam 	},

	/* file I/O functions */
	{	"openi",	SUBR,	CC65FIX xopeni 		},
	{	"openo",	SUBR,	CC65FIX xopeno 		},
	{	"openib",	SUBR,	CC65FIX xopenib 	},
	{	"openob",	SUBR,	CC65FIX xopenob 	},
	{	"close",	SUBR,	CC65FIX xclose 		},
	{	"read-char",	SUBR,	CC65FIX xrdchar 	},
	{	"peek-char",	SUBR,	CC65FIX xpkchar 	},
	{	"write-char",	SUBR,	CC65FIX xwrchar 	},
	{	"readline",	SUBR,	CC65FIX xreadline 	},

	/* system functions */
	{	"load",		SUBR,	CC65FIX xload 		},
	{	"gc",		SUBR,	CC65FIX xgc 		},
	{	"expand",	SUBR,	CC65FIX xexpand 	},
	{	"alloc",	SUBR,	CC65FIX xalloc	 	},
	{	"mem",		SUBR,	CC65FIX xmem 		},
	{	"type",		SUBR,	CC65FIX xtype 		},
	{	"exit",		SUBR,	CC65FIX xexit 		},

	{	NULL,		0,	CC65FIX NULL 		}
};
