/* xlinit.c - xlisp initialization module */
#include "xlset.h"
#include "xlinit.h"
#include "xldmem.h"
#include "xlobj.h"
#include "xlsubr.h"
#include "xlsym.h"
#include "xleval.h"
#include "xlftab.h"

NODE *true;
NODE *s_quote;
NODE *s_lambda;
NODE *s_nlambda;
NODE *s_stdin;
NODE *s_stdout;
NODE *s_tracenable;
NODE *k_rest;
NODE *k_aux;
NODE *a_subr;
NODE *a_fsubr;
NODE *a_list;
NODE *a_sym;
NODE *a_int;
NODE *a_str;
NODE *a_obj;
NODE *a_fptr;

/* xlinit - xlisp initialization routine */
void xlinit(void)
{
	struct fdef *fptr;
	NODE *sym;

	/* initialize xlisp (must be in this order) */
	xlminit();	/* initialize xldmem.c */
	xlsinit();	/* initialize xlsym.c */
	xleinit();	/* initialize xleval.c */
	xloinit();	/* initialize xlobj.c */

	/* enter the builtin functions */
	for (fptr = ftab; fptr->f_name; fptr++) xlsubr(fptr->f_name,fptr->f_type,fptr->f_fcn);

	/* enter the 't' symbol */
	true = xlsenter("t");
	true->n_symvalue = true;

	/* enter some important symbols */
	s_quote	= xlsenter("quote");
	s_lambda = xlsenter("lambda");
	s_nlambda = xlsenter("nlambda");
	k_rest = xlsenter("&rest");
	k_aux = xlsenter("&aux");

	/* enter *standard-input* and *standard-output* */
	s_stdin = xlsenter("*standard-input*");
	s_stdin->n_symvalue = newnode(FPTR);
	s_stdin->n_symvalue->n_fp = stdin;
	s_stdin->n_symvalue->n_savech = 0;
	s_stdout = xlsenter("*standard-output*");
	s_stdout->n_symvalue = newnode(FPTR);
	s_stdout->n_symvalue->n_fp = stdout;
	s_stdout->n_symvalue->n_savech = 0;

	/* enter the error traceback enable flag */
	s_tracenable = xlsenter("*tracenable*");
	s_tracenable->n_symvalue = true;

	/* enter a copyright notice into the oblist */
	sym = xlsenter("**Copyright-1984-by-David-Betz**");
	sym->n_symvalue = true;

	/* enter type names */
	a_subr	= xlsenter("SUBR");
	a_fsubr	= xlsenter("FSUBR");
	a_list	= xlsenter("LIST");
	a_sym	= xlsenter("SYM");
	a_int	= xlsenter("INT");
	a_str	= xlsenter("STR");
	a_obj	= xlsenter("OBJ");
	a_fptr	= xlsenter("FPTR");
}
