#ifndef __H_SEEN_
#define __H_SEEN_ 1

/* xlisp - a small subset of lisp */

/* system specific definitions */

/* NNODES	number of nodes to allocate in each request */
/* TDEPTH	trace stack depth */
/* FORWARD	type of a forward declaration (usually "") */
/* LOCAL	type of a local function (usually "static") */

/* for the Computer Innovations compiler */
#ifdef CI
#define NNODES		1000
#define TDEPTH		500
#endif

/* for the CPM68K compiler */
#ifdef CPM68K
#define NNODES		1000
#define TDEPTH		500
#define LOCAL
#undef NULL
#define NULL		(char *)0
#endif

/* for the DeSmet compiler */
#ifdef DESMET
#define NNODES		1000
#define TDEPTH		500
#define LOCAL
#define getc(fp)	getcx(fp)
#define EOF		-1
#endif

/* for the VAX-11 C compiler */
#ifdef vms
#define NNODES		2000
#define TDEPTH		1000
#endif

/* for the DECUS C compiler */
#ifdef decus
#define NNODES		200
#define TDEPTH		100
#endif

/* for unix compilers */
#ifdef unix
#define NNODES		200
#define TDEPTH		100
#endif

/* for the AZTEC C compiler */
#ifdef AZTEC
#define NNODES		200
#define TDEPTH		100
#define getc(fp)	getcx(fp)
#define putc(ch,fp)	aputc(ch,fp)
#define malloc		alloc
#define strchr		index
#endif

/* CC65 compiler */
#ifdef __CC65__
#define NNODES	200
#define TDEPTH	100
#define LOCAL 	static
#endif

/* default important definitions */
#ifndef NNODES
#define NNODES	200
#endif
#ifndef TDEPTH
#define TDEPTH	100
#endif
#ifndef LOCAL
#define LOCAL	static
#endif

/* useful definitions */
#define TRUE	1
#define FALSE	0

/* program limits */
#define STRMAX		100		/* maximum length of a string constant */
	
/* node types */
#define FREE	0
#define SUBR	1
#define FSUBR	2
#define LIST	3
#define SYM	4
#define INT	5
#define STR	6
#define OBJ	7
#define FPTR	8

/* node flags */
#define MARK	1
#define LEFT	2

/* string types */
#define DYNAMIC	0
#define STATIC	1

#define NODE struct node

/* symbol structure */
struct xsym {
	NODE *xsy_plist;	/* symbol plist - points to (name.plist) */
	NODE *xsy_value;	/* the current value */
};

/* subr/fsubr node structure */
struct xsubr {
	NODE *(*xsu_subr)();	/* pointer to an internal routine */
};

/* list node structure */
struct xlist {
	NODE *xl_value;	/* value at this node */
	NODE *xl_next;	/* next node */
};

/* integer node structure */
struct xint {
	int xi_int;			/* integer value */
};

/* string node structure */
struct xstr {
	int xst_type;		/* string type */
	char *xst_str;		/* string pointer */
};

/* object node structure */
struct xobj {
	NODE *xo_obclass;	/* class of object */
	NODE *xo_obdata;	/* instance data */
};

/* file pointer node structure */
struct xfptr {
	FILE *xf_fp;		/* the file pointer */
	int xf_savech;		/* lookahead character for input files */
};

/* node structure */
NODE {
	char n_type;		/* type of node */
	char n_flags;		/* flag bits */
	union {			/* value */
		struct xsym n_xsym;	/* symbol node */
		struct xsubr n_xsubr;	/* subr/fsubr node */
		struct xlist n_xlist;	/* list node */
		struct xint n_xint;	/* integer node */
		struct xstr n_xstr;	/* string node */
		struct xobj n_xobj;	/* object node */
		struct xfptr n_xfptr;	/* file pointer node */
    	} n_info;
};


/* shorthand macros for accessing node substructures */

/* symbol node */
#define n_symplist	n_info.n_xsym.xsy_plist
#define n_symvalue	n_info.n_xsym.xsy_value

/* subr/fsubr node */
#define n_subr		n_info.n_xsubr.xsu_subr

/* list node (and message node and binding node) */
#define n_listvalue	n_info.n_xlist.xl_value
#define n_listnext	n_info.n_xlist.xl_next
#define n_msg		n_info.n_xlist.xl_value
#define n_msgcode	n_info.n_xlist.xl_next
#define n_bndsym	n_info.n_xlist.xl_value
#define n_bndvalue	n_info.n_xlist.xl_next
#define n_left		n_info.n_xlist.xl_value
#define n_right		n_info.n_xlist.xl_next
#define n_ptr		n_info.n_xlist.xl_value

/* integer node */
#define n_int		n_info.n_xint.xi_int

/* string node */
#define n_str		n_info.n_xstr.xst_str
#define n_strtype	n_info.n_xstr.xst_type

/* object node */
#define n_obclass	n_info.n_xobj.xo_obclass
#define n_obdata	n_info.n_xobj.xo_obdata

/* file pointer node */
#define n_fp		n_info.n_xfptr.xf_fp
#define n_savech	n_info.n_xfptr.xf_savech


#ifndef jmp_buf
#include <setjmp.h>
#endif

extern jmp_buf *xljmpbuf;
extern jmp_buf topjmpbuf;

#endif
