/* xlprint - xlisp print routine */
#include "xlset.h"
#include "xlprin.h"
#include "xlio.h"
#include "xlsym.h"

/* local variables */
static char buf[STRMAX + 1];

/* putstr - output a string */
LOCAL void putstr(fptr, str)
NODE *fptr;
char *str;
{
	while (*str) xlputc(fptr, *str++);
}

/* putoct - output an octal byte value */
LOCAL void putoct(fptr, n)
NODE *fptr;
int n;
{
	sprintf(buf, "%03o", n);
	putstr(fptr, buf);
}

/* putstring - output a string */
LOCAL void putstring(fptr, str)
NODE *fptr;
char *str;
{
	int ch;

	/* output the initial quote */
	xlputc(fptr, '"');

	/* output each character in the string */
	while ((ch = *str++)) {

		/* check for a control character */
		if (ch < 040 || ch == '\\') {
			xlputc(fptr, '\\');
			switch (ch) {
				case '\033':
					xlputc(fptr, 'e');
					break;
				case '\n':
					xlputc(fptr, 'n');
					break;
				case '\r':
					xlputc(fptr, 'r');
					break;
				case '\t':
					xlputc(fptr, 't');
					break;
				case '\\':
					xlputc(fptr, '\\');
					break;
				default:
					putoct(fptr, ch);
					break;
			}
		} else xlputc(fptr,ch); /* output a normal character */
	}

	/* output the terminating quote */
	xlputc(fptr,'"');
}

/* putatm - output an atom */
LOCAL void putatm(fptr, tag, val)
NODE *fptr;
char *tag;
int val;
{
	sprintf(buf, "<%s: #%x>", tag, val);
	putstr(fptr, buf);
}

/* putdec - output a decimal number */
LOCAL void putdec(fptr, n)
NODE *fptr;
int n;
{
	sprintf(buf, "%d", n);
	putstr(fptr, buf);
}

/* puthex - output a hexadecimal number */
/* currently unused
LOCAL void puthex(fptr, n)
NODE *fptr;
unsigned int n;
{
	sprintf(buf, "%x", n);
	putstr(fptr, buf);
}

*/

/* xlprint - print an xlisp value */
void xlprint(fptr, vptr, flag)
NODE *fptr;
NODE *vptr;
int flag;
{
	NODE *nptr, *next;

	/* print null as the empty list */
	if (vptr == NULL) {
		putstr(fptr, "nil");
		return;
	}

	/* check value type */
	switch (vptr->n_type) {
		case SUBR:
			putatm(fptr, "Subr", vptr);
			break;
		case FSUBR:
			putatm(fptr, "FSubr", vptr);
			break;
		case LIST:
			xlputc(fptr, '(');
			for (nptr = vptr; nptr != NULL; nptr = next) {
				xlprint(fptr, nptr->n_listvalue, flag);
				if ((next = nptr->n_listnext) != NULL) {
					if (next->n_type == LIST) {
						xlputc(fptr, ' ');
					} else {
						putstr(fptr, " . ");
						xlprint(fptr, next, flag);
						break;
					}
				}
			}
			xlputc(fptr, ')');
			break;
		case SYM:
			putstr(fptr, xlsymname(vptr));
			break;
		case INT:
			putdec(fptr, vptr->n_int);
			break;
		case STR:
			if (flag)
			putstring(fptr, vptr->n_str);
			else
			putstr(fptr, vptr->n_str);
			break;
		case FPTR:
			putatm(fptr, "File", vptr);
			break;
		case OBJ:
			putatm(fptr, "Object", vptr);
			break;
		default:
			putatm(fptr, "Foo", vptr);
			break;
	}
}

/* xlterpri - terminate the current print line */
void xlterpri(fptr)
NODE *fptr;
{
	xlputc(fptr, '\n');
}

