/* xlread - xlisp expression input routine */
#include "xlset.h"
#include "xlread.h"
#include "xlio.h"
#include "xlsym.h"
#include "xleval.h"
#include "xldmem.h"
#include "xlinit.h"
#include "xlprin.h"


#ifndef DEBUG
#define DEBUG 0
#endif

/* xlload - load a file of xlisp expressions */
int xlload(name)
char *name;
{
	jmp_buf *oldjmpbuf;
	jmp_buf	loadjmpbuf;
	NODE *oldstk, fptr, val;
	char fname[50]; /* 666 danger will robinson! PLEASE FIX ME! */
	FILE *fp;

	/* add the default extension */
	strcpy(fname, name); strcat(fname, ".lsp");

	/* open the file */
	if ((fp = fopen(fname,"r")) == NULL)
	return (FALSE);

	/* create a new stack frame */
	oldstk = xlsave(&fptr, &val, NULL);

	/* allocate a file node */
	fptr.n_ptr = newnode(FPTR);
	fptr.n_ptr->n_fp = fp;
	fptr.n_ptr->n_savech = 0;

	/* setup to trap errors */
	oldjmpbuf = xljmpbuf;
	xljmpbuf = (jmp_buf *)(&loadjmpbuf);
	if (setjmp(loadjmpbuf)) { /* was if (setjmp(xljmpbuf = loadjmpbuf)) {  */
		fclose(fp);
		xljmpbuf = oldjmpbuf;
		longjmp(*xljmpbuf, 1); /* was longjmp((xljmpbuf = oldjmpbuf), 1); */
	}

	/* read and evaluate each expression in the file */
	while (xlread(fptr.n_ptr, &val.n_ptr)) 
#if DEBUG

	{
		val.n_ptr = 
#endif

		xleval(val.n_ptr);
#if DEBUG
		xlprint(s_stdout->n_symvalue, val.n_ptr, TRUE);
		xlterpri(s_stdout->n_symvalue);
	}
#endif

	/* restore error trapping context and previous stack frame */
	xljmpbuf = oldjmpbuf;
	xlstack = oldstk;

	/* close the file */
	fclose(fp);

	/* return successfully */
	return (TRUE);
}

/* issym - check whether a character if valid in a symbol name */
LOCAL int issym(ch)
int ch;
{
	if (ch <= ' ' || ch == '(' || ch == ')' || ch == ';' ||  ch == '.' || ch == '"' || ch == '\'') return (FALSE);
	return (TRUE);
}

/* nextch - look at the next non-blank character */
LOCAL int nextch(fptr)
NODE *fptr;
{
	int ch;

	/* return and save the next non-blank character */
	while ((ch = xlpeek(fptr)) != EOF && isspace(ch)) xlgetc(fptr);
	return (ch);
}

/* badeof - unexpected eof */
LOCAL void badeof()
{
	xlfail("unexpected EOF");
}

/* checkeof - get a character and check for end of file */
LOCAL int checkeof(fptr)
NODE *fptr;
{
	int ch;

	if ((ch = xlgetc(fptr)) == EOF) badeof();
	return (ch);
}


/* pstring - parse a string */
LOCAL NODE *pstring(fptr)
NODE *fptr;
{
	NODE *oldstk, val;
	char sbuf[STRMAX + 1];
	int ch, i, d1, d2, d3;

	/* create a new stack frame */
	oldstk = xlsave(&val, NULL);

	/* skip the opening quote */
	xlgetc(fptr);

	/* loop looking for a closing quote */
	for (i = 0; i < STRMAX && (ch = checkeof(fptr)) != '"'; i++) {
		switch (ch) {
			case EOF:
				badeof();

			case '\\':
				switch (ch = checkeof(fptr)) {

					case 'e':
						ch = '\033';
						break;

					case 'n':
						ch = '\n';
						break;

					case 'r':
						ch = '\r';
						break;

					case 't':
						ch = '\t';
						break;

					default:
						if (ch >= '0' && ch <= '7') {
							d1 = ch - '0';
							d2 = checkeof(fptr) - '0';
							d3 = checkeof(fptr) - '0';
							ch = (d1 << 6) + (d2 << 3) + d3;
						}
						break;
				}
		}
		sbuf[i] = ch;
	}
	sbuf[i] = 0;

	/* initialize the node */
	val.n_ptr = newnode(STR);
	val.n_ptr->n_str = strsave(sbuf);
	val.n_ptr->n_strtype = DYNAMIC;

	/* restore the previous stack frame */
	xlstack = oldstk;

	/* return the new string */
	return (val.n_ptr);
}

/* pname - parse a symbol name */
LOCAL NODE *pname(fptr)
NODE *fptr;
{
	char sname[STRMAX + 1];
	NODE *val;
	int i;

	/* get symbol name */
	for (i = 0; i < STRMAX && issym(xlpeek(fptr)); ) sname[i++] = xlgetc(fptr);
	sname[i] = 0;

	/* check for a number or enter the symbol into the oblist */
	return (isnumber(sname, &val) ? val : xlenter(sname, DYNAMIC));
}

/* pcomment - parse a comment */
LOCAL void pcomment(fptr)
NODE *fptr;
{
	int ch;

	/* skip to end of line */
	while ((ch = checkeof(fptr)) != EOF && ch != '\n') ;
}

/* 666 fixes for a poor design 666 */
LOCAL NODE *plist(NODE *);
LOCAL NODE *pquote(NODE *);


/* parse - parse an xlisp expression */
LOCAL int parse(fptr, pval)
NODE *fptr;
NODE **pval;
{
	int ch;

	/* keep looking for a node skipping comments */
	for(;;) {

		/* check next character for type of node */
		ch = nextch(fptr);
		switch(ch) {
			case EOF:
				return (FALSE);

			case '\'': /* a quoted expression */
				*pval = pquote(fptr); /* 666 */
				return (TRUE);

			case '(': /* a sublist */
				*pval = plist(fptr); /* 666 */
				return (TRUE);

			case ')': /* closing paren - shouldn't happen */
				xlfail("extra right paren");

			case '.': /* dot - shouldn't happen */
				xlfail("misplaced dot");

			case ';': /* a comment */
				pcomment(fptr);
				break;

			case '"': /* a string */
				*pval = pstring(fptr);
				return (TRUE);

			default: /* a name */
				if (issym(ch)) {
					*pval = pname(fptr);
				} else xlfail("invalid character");
				return (TRUE);
		}
	}
}

/* plist - parse a list */
LOCAL NODE *plist(fptr)
NODE *fptr;
{
	NODE *oldstk, val, *lastnptr, *nptr;
	int ch;

	/* increment the nesting level */
	xlplevel += 1;

	/* create a new stack frame */
	oldstk = xlsave(&val, NULL);

	/* skip the opening paren */
	xlgetc(fptr);

	/* keep appending nodes until a closing paren is found */
	lastnptr = NULL;
	for (lastnptr = NULL; (ch = nextch(fptr)) != ')'; lastnptr = nptr) {

		/* check for end of file */
		if (ch == EOF) badeof();

		/* check for a dotted pair */
		if (ch == '.') {

			/* skip the dot */
			xlgetc(fptr);

			/* make sure there's a node */
			if (lastnptr == NULL) xlfail("invalid dotted pair");

			/* parse the expression after the dot */
			if (!parse(fptr, &lastnptr->n_listnext)) badeof(); /* 666 */

			/* make sure its followed by a close paren */
			if (nextch(fptr) != ')') xlfail("invalid dotted pair");

			/* done with this list */
			break;
		}

		/* allocate a new node and link it into the list */
		nptr = newnode(LIST);
		if (lastnptr == NULL) {
			val.n_ptr = nptr;
		} else lastnptr->n_listnext = nptr;

		/* initialize the new node */
		if (!parse(fptr, &nptr->n_listvalue)) badeof();
	}

	/* skip the closing paren */
	xlgetc(fptr);

	/* restore the previous stack frame */
	xlstack = oldstk;

	/* decrement the nesting level */
	xlplevel -= 1;

	/* return successfully */
	return (val.n_ptr);
}

/* pquote - parse a quoted expression */
LOCAL NODE *pquote(fptr)
NODE *fptr;
{
	NODE *oldstk, val;

	/* create a new stack frame */
	oldstk = xlsave(&val, NULL);

	/* skip the quote character */
	xlgetc(fptr);

	/* allocate two nodes */
	val.n_ptr = newnode(LIST);
	val.n_ptr->n_listvalue = s_quote;
	val.n_ptr->n_listnext = newnode(LIST);

	/* initialize the second to point to the quoted expression */
	if (!parse(fptr, &val.n_ptr->n_listnext->n_listvalue)) badeof(); /* 666 */

	/* restore the previous stack frame */
	xlstack = oldstk;

	/* return the quoted expression */
	return (val.n_ptr);
}

/* xlread - read an xlisp expression */
int xlread(fptr, pval)
NODE *fptr;
NODE **pval;
{
	/* initialize */
	xlplevel = 0;

	/* parse an expression */
	return (parse(fptr, pval));
}


/* isnumber - check if this string is a number */
int isnumber(str, pval)
char *str;
NODE **pval;
{
	char *p;
	int d;

	/* initialize */
	p = str;
	d = 0;

	/* check for a sign */
	if (*p == '+' || *p == '-') p++;

	/* check for a string of digits */
	while (isdigit(*p)) {
		p++;
		d++;
	}

	/* make sure there was at least one digit and this is the end */
	if (d == 0 || *p) return (FALSE);

	/* convert the string to an integer and return successfully */
	*pval = newnode(INT);
	(*pval)->n_int = atoi(*str == '+' ? ++str : str);
	return (TRUE);
}

